import java.awt.Color; // To color our distributors
/**
 * Distributor -- takes orders from Market to Warehouse,
 * fills them, and returns with product.
 **/
public class Distributor extends DEAgent {
    
  /////// Constants for Messages
  public static final int MARKET_ARRIVE = 0;
  public static final int MARKET_LEAVE = 1;
  public static final int WAREHOUSE_ARRIVE = 2;
  
  /** AmountOrdered so-far */
  int amountOrdered;
  
  // Methods
    
  public int newOrders(){
    // Between 5 and 25, uniform
    return randNumGen.nextInt(21)+5;
  }
  
  public double timeToDeliver(){
    // On average 5 days to deliver, normal distr.
    return validTime(randNumGen.nextGaussian()*5);}
  
  public double tripTime(){
    // On average 2 days to travel between market and warehouse
    return validTime(randNumGen.nextGaussian()*2);}
    
  /**
   * Initialize a distributor.
   * Start in the market, taking orders, then
   * schedule arrival at the warehouse.
   **/
  public void init(Simulation thisSim){
    //First, do the normal stuff
    super.init(thisSim);
    this.setPenDown(false); // Pen up
    this.setBodyColor(Color.blue); // Go Blue!
    
    // Show the distributor in the market
    this.moveTo(600,460); // At far right
    // Get the orders, and set off for the warehouse
    amountOrdered = this.newOrders();
    ((DESimulation) thisSim).addEvent(
              new SimEvent(this,tripTime(),WAREHOUSE_ARRIVE));
  }
  
  /** Are we ready to be unlocked? */
  public boolean isReady(Resource res) {
    // Is the amount in the factory more than our orders?
    return ((FactorySimulation) simulation).getFactory().amountAvailable() >= amountOrdered;}
  
  /**
   * I've been unblocked!
   * @param resource the desired resource
   **/
  public void unblocked(Resource resource){
    super.unblocked(resource);
    
    // Consume the resource for the orders
    ((FactoryProduct) resource).consume(amountOrdered); // Zero time to load?
    ((DESimulation) simulation).log(this.getName()+"\t Gathered product for orders of \t"+amountOrdered);
    // Schedule myself to arrive at the Market
    ((DESimulation) simulation).addEvent(
              new SimEvent(this,tripTime(),MARKET_ARRIVE));
  }
  
  /**
   * Process an event.
   * Default is to do nothing with it.
   **/
  public void processEvent(int message){
      switch(message){
        case MARKET_ARRIVE:
          // Show the distributor at the market, far left
          ((DESimulation) simulation).log(this.getName()+"\t Arrived at market");
          this.moveTo(210,460);
          // Schedule time to deliver
          ((DESimulation) simulation).addEvent(
              new SimEvent(this,timeToDeliver(),MARKET_LEAVE));
          break;
        case MARKET_LEAVE:
          // Show the distributor at the market, far right
          ((DESimulation) simulation).log(this.getName()+"\t Leaving market");
          this.moveTo(600,460);
          // Get the orders, and set off for the warehouse
          amountOrdered = this.newOrders();
          ((DESimulation) simulation).addEvent(
              new SimEvent(this,tripTime(),WAREHOUSE_ARRIVE));
           break;
        case WAREHOUSE_ARRIVE:
          // Show the distributor at the warehouse
          ((DESimulation) simulation).log(this.getName()+"\t Arrived at warehouse");
          this.moveTo(600,50);
          // Is there enough product available?
          FactoryProduct factory = ((FactorySimulation) simulation).getFactory();
          if (factory.amountAvailable() >= amountOrdered)
          {    
            // Consume the resource for the orders
            factory.consume(amountOrdered); // Zero time to load?
            ((DESimulation) simulation).log(this.getName()+"\t Gathered product for orders of \t"+amountOrdered);
            // Schedule myself to arrive at the Market
            ((DESimulation) simulation).addEvent(
                   new SimEvent(this,tripTime(),MARKET_ARRIVE));
          }
          else {// We have to wait until more product arrives!
            ((DESimulation) simulation).log(this.getName()+"\t is blocking");
            waitFor(((FactorySimulation) simulation).getFactory());}
        break;
    }}
  
    ////////////////////////////// Constructors ////////////////////////
  // Copy this section AS-IS into subclasses, but rename Agent to 
  // Your class.
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned)
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public Distributor (ModelDisplay modelDisplayer,Simulation thisSim)
  {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer, thisSim);
  }
  
  /** Constructor that takes the x and y and a model
   * display to draw it on
   * @param x the starting x position
   * @param y the starting y position
   * @param modelDisplayer the thing that displays the model
   * @param thisSim my simulation
   */
  public Distributor (int x, int y, ModelDisplay modelDisplayer, 
               Simulation thisSim) 
  {
    // let the parent constructor handle it
    super(x,y,modelDisplayer,thisSim);
  }
}