/**
 * Resource class -- manages agents requesting a resource
 **/
public class Resource {
  
  /// Fields
  
  /** Amount of resource available */
  protected int amount;
  
  /** Queue of who's waiting for resource */
  protected Queue blocked; 
  
  //// Constructors ////
  /**
   * Start out with no resource and an empty queue
   **/
  public Resource(){
    amount = 0;
    blocked = new Queue();
  }
  
  /** Amount available */
  public int amountAvailable() {return amount;}
  
  /** Consume resource */
  public void consume(int amtDesired){
    amount = amount - amtDesired;
  }
  
  /**
   * Add more produced resource.
   * Is there enough to unblock the first 
   * Agent in the Queue?
   **/
  public void add(int production) {
    amount = amount + production;
    
    if (!blocked.empty()){
      // Ask the next Agent in the queue if it can be unblocked
      DEAgent topOne = (DEAgent) blocked.peek();
      // Is it ready to run given this resource?
      if (topOne.isReady(this)) {
        // Remove it from the queue
        topOne = (DEAgent) blocked.pop();
        // And tell it its unblocked
        topOne.unblocked(this);
      }
    }
  }
  
  /**
   * addToList -- add this agent to
   * this resource's waiting list.
   **/
  public void addToList(DEAgent agent){
    blocked.push(agent);
  }
}
